def span(predicate, iterable):
    out = [[]]
    current = out[0]
    for i in iterable:
        if not predicate(i):
            current.append(i)
        else:
            current = []
            out.append(current)

    return out

def get_keys(keyfile):
    import zmq
    import os, errno, json
    if not os.path.exists(keyfile):
        keypair = zmq.curve_keypair()
        path = os.path.dirname(os.path.abspath(keyfile))
        try:
            os.mkdir(path, 0700)
        except OSError as e:
            if not (e.errno == errno.EEXIST and os.path.isdir(path)):
                raise

        with open(keyfile, 'w', 0600) as k:
            json.dump(keypair, k)

        return [True] + list(keypair)
    else:
        with open(keyfile, 'r') as k:
            return [False] + map(str, json.load(k))

def fork_upstream_proxy(ipcaddr, router_url, router_ident, (pubkey, seckey)):
    import zmq
    from zmq.devices import ProcessDevice
    pd = ProcessDevice(zmq.QUEUE, zmq.ROUTER, zmq.DEALER)
    pd.bind_in(ipcaddr)
    pd.connect_out(router_url)
    pd.setsockopt_out(zmq.IDENTITY, pubkey)
    pd.setsockopt_out(zmq.CURVE_PUBLICKEY, pubkey)
    pd.setsockopt_out(zmq.CURVE_SECRETKEY, seckey)
    pd.setsockopt_out(zmq.CURVE_SERVERKEY, router_ident)
    pd.start()
